// $Id: Editor.cc,v 1.24 1999/12/13 22:49:49 grumbel Exp $
//
// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <cstdio>
#include <ClanLib/clanlib.h>

#include "../algo.hh"
#include "../globals.hh"
#include "../PingusError.hh"
#include "../PingusMessageBox.hh"
#include "../PingusResource.hh"
#include "../PingusGame.hh"
#include "Editor.hh"

Editor::Editor ()
{
  plf = 0;
  psm = 0;
  start_x_pos = 0;
  start_y_pos = 0;
  last_level = pingus_homedir + "levels/";
  always_animate = false;
  x_offset = 0;
  y_offset = 0;
  font = CL_Res_Font::load("Fonts/smallfont_h",*PingusResource::get("global.dat"));
  status_line.set_current_objs(&current_objs);
  bg.r = 0.3;
  bg.g = 0.3;
  bg.b = 0.3;
}

Editor::~Editor ()
{
}

void
Editor::edit (string filename)
{
  bool quit = false;
  int mouse_x = -1;
  int mouse_y = -1;

  load_level (filename);
    
  while (!quit) {
    // cout << "Iterator: " << current_objs << endl;
    CL_System::keep_alive();

    if (always_animate)
      draw();

    if (CL_Mouse::left_pressed()) {
      EditorObjIter c_obj = get_current_obj();
      
      if (c_obj == 0) {
	if (!CL_Keyboard::get_keycode(CL_Keyboard::KEY_LSHIFT))
	  delete_selection();
      } else {
	if (object_selected(c_obj)) {
	  if (mouse_moved()) {
	    move_obj();
	  } else {
	    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_LSHIFT)) {
	      unmark(c_obj);
	    } else {
	      delete_selection();
	      current_objs.push_back(c_obj);
	    }
	  }
	} else {
	  if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_LSHIFT)) {
	    current_objs.push_back(c_obj);
	  } else {
	    delete_selection();
	    current_objs.push_back(c_obj);
	}
	  while(CL_Mouse::left_pressed())
	    CL_System::keep_alive();
	}
      }
      draw();
    }

    if (CL_Mouse::middle_pressed()) {
      rect_get_current_objs();
    }

    if (CL_Mouse::right_pressed()) {
      scroll();
    }

    if (key_pressed(CL_Keyboard::KEY_F3)) {
      bg.r += 0.1;
      bg.g += 0.1;
      bg.b += 0.1;
      
      if (bg.r > 1.0) 
	bg.r = bg.g = bg.b = 0.0;
    }

    if (key_pressed(CL_Keyboard::KEY_F4)) {
      PingusGame game;
      string levelfile = save_tmp_level();

      game.start(levelfile + ".plf",
		 levelfile + ".psm");     
      draw();
    }

    if (key_pressed(CL_Keyboard::KEY_F2)) {
      always_animate = !always_animate;
    }

    if (key_pressed(CL_Keyboard::KEY_ESCAPE)) {
      // FIXME: tmpnam() is POSIX
      save_tmp_level ();
      quit = true;
    }

    if (key_pressed(CL_Keyboard::KEY_F7)) {
      save_tmp_level();

      checkpoint = string(tmpnam(0)) + ".pingus";
      
      cout << "Setting checkpoint: " << checkpoint << endl;

      save_level(checkpoint);    
    }

    if (key_pressed(CL_Keyboard::KEY_F8)) {
      save_tmp_level();
      if (!checkpoint.empty()) {
	cout << "Restoring checkpoint: " << checkpoint << endl;
	load_level(checkpoint);
      } else {
	cout << "No checkpoint set, no restoring done. " << endl;
      }
    }

    if (key_pressed(CL_Keyboard::KEY_INSERT)) {
      save_tmp_level ();

      EditorObj* obj;
      try {
	obj = object_selector.get_obj(x_offset, y_offset);
      }
      catch (CL_Error err) {
	cout << "Error caught from ClanLib: " << err.message << endl;
	// delete obj;
	obj = 0;
      }
      
      if (obj) {
	editor_objs.push_back(obj);
      } else {
	cout << "Something went wrong while inserting" << endl;
      }
      draw();
    }
    
    if (key_pressed(CL_Keyboard::KEY_F5)) {
      string str;
      cout << "File to load: " << flush;
      str = read_string("Input level to load (without the .plf ending!):");
      if (!str.empty()) {
	try {
	  last_level = str;
	  load_level(str);
	}
	catch (PingusError err) {
	  PingusMessageBox(" PingusError: " + err.message);
	}
	draw();
      }
    }
    
    if (key_pressed(CL_Keyboard::KEY_A)) {
      delete_selection();
      for(EditorObjIter i = editor_objs.begin(); i != editor_objs.end(); ++i) {
	current_objs.push_back(i);
      }
    }

    if (key_pressed(CL_Keyboard::KEY_F6)) {
      cout << "Saving level, input filename" << endl;
      string str = read_string("Input filename to save the file (without .plf!)");
      if (!str.empty()) {
	save_level (str);
      }
    }
    
    if (key_pressed(CL_Keyboard::KEY_F10)) {
      background.desc.res_name = read_string("Input Background: ", background.desc.res_name);
      draw();
    }

    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_DELETE)) {
      save_tmp_level ();
      for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {
	// surfaces.erase(*i);
	editor_objs.erase(*i);
      }
      current_objs.erase(current_objs.begin(), current_objs.end());
      draw();
    }
    
    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_PAGEDOWN)) {
      lower_current_objs();
      if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_RSHIFT)) {
	for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {      
	  (**i)->z_pos = -100;
	}
      }
      draw();
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_PAGEUP)) {
      raise_current_objs();
      if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_RSHIFT)) {
	for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {      
	  (**i)->z_pos = 100;
	}
      }
      draw();
    }

    if (key_pressed(CL_Keyboard::KEY_F9)) {
      string tmp_str;
      char str[1024];
      int  theight, twidth;

      sprintf(str, "%d", width);
      tmp_str = read_string("Input Width: ", str);
      if (sscanf(tmp_str.c_str(), "%d", &twidth) == 1) {
	width = twidth;
      }

      sprintf(str, "%d", height);
      tmp_str = read_string("Input Height: ", str);
      if (sscanf(tmp_str.c_str(), "%d", &theight) == 1) {
	height = theight;
      }
      draw();
    }
    
    if (key_pressed(CL_Keyboard::KEY_KP_PLUS)) {
      for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {      
	editor_objs.insert(*i, (**i)->duplicate());
	cout << "Editor: Inserting Object" << endl;
      }
    }

    if (mouse_x != CL_Mouse::get_x()
	|| mouse_y != CL_Mouse::get_y()) {
      draw();
      mouse_x = CL_Mouse::get_x();
      mouse_y = CL_Mouse::get_y();
    }
    
    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_RSHIFT)) {
      move_offset = 10;
    } else {
      move_offset = 1;
    }
    
    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_LEFT)) {
      for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i)
	(**i)->x_pos -= move_offset;
      draw();
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_RIGHT)) {
      for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i)
	(**i)->x_pos += move_offset;
      draw();
    }
    
    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_UP)) {
      for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i)
	(**i)->y_pos -= move_offset;
      draw();
    } else if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_DOWN)) {
      for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i)
	(**i)->y_pos += move_offset;
      draw();
    }
  }
}

Editor::EditorObjIter
Editor::get_current_obj()
{
  EditorObjIter c_obj = 0;

  for (EditorObjIter i = editor_objs.begin(); i != editor_objs.end(); ++i) {
    if ((*i)->mouse_over(x_offset, y_offset)) {
      c_obj = i;
    }
  }
  return c_obj;
}

void
Editor::draw ()
{
  draw_noflip();
  status_line.draw(x_offset, y_offset);
  CL_Display::flip_display(true);
}

void 
Editor::draw_noflip()
{
  CL_Display::clear_display();
  CL_Display::fill_rect(0 + x_offset,0 + y_offset,
			width + x_offset, height + y_offset,
			bg.r, bg.g, bg.b, 1.0);
  
  // for (SurDataIter i = surfaces.begin(); i != surfaces.end(); ++i) {
  //   i->surface->put_screen(i->x_pos + x_offset, i->y_pos + y_offset);
  // }
  for (EditorObjIter i = editor_objs.begin(); i != editor_objs.end(); ++i) {
    (*i)->draw_offset(x_offset, y_offset);
  }

  for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {
    (**i)->draw_mark_offset(x_offset, y_offset);
  } 
}

void
Editor::move_obj()
{
  int mouse_x = CL_Mouse::get_x();
  int mouse_y = CL_Mouse::get_y();
  
  while (CL_Mouse::left_pressed()) {
    for (CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {
      (**i)->x_pos += CL_Mouse::get_x() - mouse_x;
      (**i)->y_pos += CL_Mouse::get_y() - mouse_y;
    }

    draw();
    mouse_x = CL_Mouse::get_x();
    mouse_y = CL_Mouse::get_y();
    CL_System::keep_alive();
  }
}

void
Editor::scroll()
{
  int mouse_x = CL_Mouse::get_x();
  int mouse_y = CL_Mouse::get_y();

  while (CL_Mouse::right_pressed()) {
    x_offset -= (CL_Mouse::get_x() - mouse_x)/5;
    y_offset -= (CL_Mouse::get_y() - mouse_y)/5;
    draw();
    CL_System::keep_alive();
  }
}

void
Editor::load_level (string filename)
{
  cout << "Clearing current level..." << endl;

  if (plf) delete plf;
  if (psm) delete psm;
  
  cout << "Loading new level: " << filename << endl;
  psm = new PSMParser;
  plf = new PLF (filename + ".plf");
  
  psm->parse (filename + ".psm");

  psm->load_surfaces();

  current_objs.erase(current_objs.begin(), current_objs.end());
  editor_objs.erase(editor_objs.begin(), editor_objs.end());

  {
    vector<surface_data>  temp_surfaces = psm->get_surfaces();
    vector<entrance_data> temp_entraces = plf->get_entrance();
    vector<hotspot_data>  temp_hotspots = plf->get_hotspot();
    vector<exit_data>     temp_exits    = plf->get_exit();
    vector<liquid_data>   temp_liquid   = plf->get_liquids();
    vector<trap_data>     temp_traps    = plf->get_traps();

    for(vector<surface_data>::iterator i = temp_surfaces.begin(); i != temp_surfaces.end(); ++i)
      editor_objs.push_back(EditorObj::create(*i));
    
    for(vector<entrance_data>::iterator i = temp_entraces.begin(); i != temp_entraces.end(); ++i)
      editor_objs.push_back(EditorObj::create(*i));

    for(vector<exit_data>::iterator i = temp_exits.begin(); i != temp_exits.end(); ++i)
      editor_objs.push_back(EditorObj::create(*i));

    for(vector<hotspot_data>::iterator i = temp_hotspots.begin(); i != temp_hotspots.end(); ++i)
      editor_objs.push_back(EditorObj::create(*i));

    for(vector<liquid_data>::iterator i = temp_liquid.begin(); i != temp_liquid.end(); ++i)
      editor_objs.push_back(EditorObj::create(*i));

    for(vector<trap_data>::iterator i = temp_traps.begin(); i != temp_traps.end(); ++i)
      editor_objs.push_back(EditorObj::create(*i));

    description = plf->get_description();
    levelname   = plf->get_levelname();
    level_time  = plf->get_time();
    number_to_save = plf->get_number_to_save();
    number_of_pingus = plf->get_pingus();
    height      = plf->get_height();
    width       = plf->get_width();
    background  = plf->get_bg();
    start_x_pos = plf->get_startx();
    start_y_pos = plf->get_starty();
    actions     = plf->get_buttons();
  }
  // editor_objs.reverse();
}

void
Editor::save_level (string filename)
{
  cout << "Saving Levelfile..." << endl;

  ofstream plf_out;
  ofstream psm_out;

  plf_out.open((filename + ".plf").c_str());
  psm_out.open((filename + ".psm").c_str());

  if (!plf_out) {
    cout << "Couldn't open plf: " << filename << endl;
    return;
  }

  if (!psm_out) {
    cout << "Couldn't open psm: " << filename << endl;
    return;
  }

  // FIXME: we need some error checking
  
  plf_out << "global {\n"
	  << "  start_x_pos = " << start_x_pos << ";\n"
	  << "  start_y_pos = " << start_y_pos << ";\n"
	  << "  levelname = \"" << levelname << "\";\n" 
	  << "  description = \"" << description << "\";\n"
	  << "  number_of_pingus = " << number_of_pingus << ";\n"
	  << "  number_to_save = " << number_to_save << ";\n"
	  << "}\n"
	  << endl;

  plf_out << "ground {\n"
	  << "  maptype = \"SPOT\";\n"
	  << "  data    = (file)\"" << basename(filename + ".psm") << "\";\n"
	  << "  width   = " << width << ";\n"
	  << "  height  = " << height << ";\n"
	  << "  colmap  = (auto);\n"
	  << "}\n"
	  << endl;
  
  plf_out << "background {\n"
	  << "  image = (resource)\"" << background.desc.res_name << "\";\n"
	  << "  dim = \"" << background.dim << "\";\n"
	  << "  scroll_x = " << background.scroll_x << ";\n"
	  << "  scroll_y = " << background.scroll_x << ";\n"
	  << "}\n"
	  << endl;

  // Printing actions to file
  plf_out << "buttons {\n";
  for (vector<button_data>::iterator i=actions.begin(); i != actions.end(); ++i) {
    plf_out << "  " << (*i).name << " = " << (*i).number_of << ";" << endl;
  }
  plf_out << "}\n" << endl;

  for (EditorObjIter i = editor_objs.begin(); i != editor_objs.end(); ++i) {
    (*i)->save(&plf_out, &psm_out);
  }
  
  plf_out << "/* EOF */" << endl;

  psm_out.close();
  plf_out.close();
  cout << "Saveing finished" << endl;
}

bool
Editor::key_pressed(int key_no)
{
  if (CL_Keyboard::get_keycode(key_no)) {
    while (CL_Keyboard::get_keycode(key_no))
      CL_System::keep_alive();
    return true;
  } else {
    return false;
  }
}

string
Editor::read_string (string prefix, string default_str)
{
  string str;

  if (default_str.empty()) {
    if (last_level.empty()) {
      str = pingus_homedir + "levels/";
    } else {
      str = last_level;
    }
  } else {
    str = default_str;
  }
      
  CL_Display::clear_display();
  font->print_left(20, 20, prefix.c_str());
  font->print_left(20, 40, str.c_str());
  CL_Display::flip_display();
  
  CL_InputBuffer*   key_buffer = CL_Input::keyboards[0]->get_buffer(0);
  CL_Key key;

  key_buffer->clear();

  while (true) { 
    CL_System::keep_alive();

    if (key_buffer->peek_key().state != CL_Key::Pressed) {
      key_buffer->get_key();
      continue;
    }

    key = key_buffer->get_key();
  
    if (key.id == CL_Keyboard::KEY_ENTER) {
      break;
    } else if (key.id == CL_Keyboard::KEY_ESCAPE) {
      while(CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE)) {
	CL_System::keep_alive();
      }
      return "";
    } else if (key.id == CL_Keyboard::KEY_DELETE
	       || key.id == CL_Keyboard::KEY_BACKSPACE) {
      if (!str.empty())
	str = str.substr(0, str.size() - 1); 
    } else if (key.id == CL_Keyboard::KEY_TAB) {
      cout << "Tab completion not implemented" << endl;
    } else {
      if (key.ascii > 0)
	str += key.ascii;
    }

    /*    cout << "Current char: " << key.ascii 
	 << " " << key.state << endl;
    cout << str << endl;
    */
    CL_Display::clear_display();
    font->print_left(20, 20, prefix.c_str());
    font->print_left(20, 40, str.c_str());
    CL_Display::flip_display();
  }
  key_buffer->clear();
  return str;
}

bool
Editor::object_selected(EditorObjIter c_obj)
{
  for(CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {
    if (*c_obj == **i) {
      return true;
    }
  }
  return false;
}

void
Editor::delete_selection()
{
  current_objs.erase(current_objs.begin(), current_objs.end());  
}

void
Editor::unmark(EditorObjIter c_obj)
{
  for(CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {
    if (*c_obj == **i) {
      current_objs.erase(i);
      return;
    }
  }
}

bool
Editor::mouse_moved()
{
  int mouse_x = CL_Mouse::get_x();
  int mouse_y = CL_Mouse::get_y();

  while(mouse_x + 1 >= CL_Mouse::get_x() && mouse_x - 1 <= CL_Mouse::get_x()
	&& mouse_y + 1 >= CL_Mouse::get_y() && mouse_y - 1 <= CL_Mouse::get_y()) {
    CL_System::keep_alive();
    if (!CL_Mouse::left_pressed())
      return false;
  }
  return true;
}

void
Editor::raise_current_objs()
{
  for(CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {
    if (raise_obj(*i)) {
      ++(*i);
    }
  }
}

void 
Editor::lower_current_objs()
{
  for(CurrentObjIter i = current_objs.begin(); i != current_objs.end(); ++i) {
    if (lower_obj(*i)) {
      --(*i);
    }
  }
}

bool
Editor::lower_obj(EditorObjIter obj)
{
  cout << "Raising Obj..." << endl;
  if (obj != editor_objs.begin()) {
    EditorObjIter prev = obj;
    --prev;
    //cout << "swapping: " << *prev << " and " << *obj << endl;
    swap(*prev, *obj);
    //cout << "swapping done: " << *prev << " and " << *obj << endl;
    return true;
  }
  return false;
}

bool
Editor::raise_obj(EditorObjIter obj)
{
  EditorObjIter next = obj;
  ++next;
  
  if (next != editor_objs.end()) {
    //cout << "swapping: " << *next << " and " << *obj << endl;
    swap(*next, *obj);
    //cout << "swapping done: " << *next << " and " << *obj << endl;
    return true;
  }
  return false;
}

string
Editor::basename(string filename)
{
  const char* str = filename.c_str();
  int i;
  cout << "Getting basename of: " << str << endl;
  for(i = filename.size() - 1; i >= 0; --i) {
    //cout << str[i] << endl;
    if (*(str + i) == '/') {
	break;
    }
  }
  cout << "Basename: " << (str+i + 1) << endl;
  return (str+i + 1);
}

void
Editor::rect_get_current_objs()
{
  int mouse_x = CL_Mouse::get_x();
  int mouse_y = CL_Mouse::get_y();

  int x1 = CL_Mouse::get_x();
  int y1 = CL_Mouse::get_y();
  int x2 = CL_Mouse::get_x();
  int y2 = CL_Mouse::get_y();

  while(CL_Mouse::middle_pressed()) {
    CL_System::keep_alive();
    draw_noflip();
    
    if (mouse_x > CL_Mouse::get_x()) {
      x1 = CL_Mouse::get_x();
      x2 = mouse_x;
    } else {
      x1 = mouse_x;
      x2 = CL_Mouse::get_x();
    }

    if (mouse_y > CL_Mouse::get_y()) {
      y1 = CL_Mouse::get_y();
      y2 = mouse_y;
    } else {
      y1 = mouse_y;
      y2 = CL_Mouse::get_y();
    }

    CL_Display::fill_rect(x1, y1, x2, y2,
			  0.0, 0.0, 1.0, 0.3);

    CL_Display::flip_display();
  }

  delete_selection();

  for(EditorObjIter i = editor_objs.begin(); i != editor_objs.end(); ++i) {
    if ((*i)->is_in_rect(x1 - x_offset, y1 - y_offset, 
			 x2 - x_offset, y2 - y_offset))
      current_objs.push_back(i);
  }
}

string
Editor::save_tmp_level ()
{
  string filename = string(tmpnam(0)) + ".pingus";
  
  cout << "Saving level to: " << filename << endl;
  
  save_level(filename.c_str());

  return filename;
}

void
Editor::interactive_load()
{
  string str;
  cout << "File to load: " << flush;
  str = read_string("Input level to load (without the .plf ending!):");
  if (!str.empty()) {
    try {
      last_level = str;
      edit(str);
    }
    catch (PingusError err) {
      PingusMessageBox(" PingusError: " + err.message);
      interactive_load();
    }
    draw();
  }
}

/***********************************************
$Log: Editor.cc,v $
Revision 1.24  1999/12/13 22:49:49  grumbel
Fixed a bug in the 'buttons' section

Revision 1.23  1999/12/12 03:05:30  grumbel
Added some cvs keywords, misc other fixes
************************************************/

/* EOF */
