// $Id: ObjectSelector.cc,v 1.5 1999/12/12 03:05:30 grumbel Exp $
//
// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "../PingusResource.hh"

#include "ObjectSelector.hh"

ObjectSelector::ObjectSelector()
{
  last_object = "GroundPieces/Snow/snowman";
  font = CL_Res_Font::load("Fonts/smallfont_h",*PingusResource::get("global.dat"));  
}

ObjectSelector::~ObjectSelector()
{
  
}
  
EditorObj* 
ObjectSelector::get_obj(int x_off, int y_off)
{
  x_offset = x_off;
  y_offset = y_off;

  return select_obj_type();
}

EditorObj* 
ObjectSelector::get_trap()
{
  trap_data trap;
  bool have_name = false;
  trap.x_pos = CL_Mouse::get_x() - x_offset;
  trap.y_pos = CL_Mouse::get_y() - y_offset;
  trap.z_pos = 0;

  CL_Display::clear_display();
  font->print_left(20, 20, "Select a trap");
  font->print_left(20, 50, "1 - guillotine");
  font->print_left(20, 70, "2 - hammer");
  font->print_left(20, 90, "3 - spike");
  font->print_left(20,110, "4 - laser_exit");
  font->print_left(20,130, "5 - fake_exit");
  font->print_left(20,150, "6 - smasher");
  CL_Display::flip_display();

  while (!have_name) {
    switch (read_key()) {

    case CL_Keyboard::KEY_1:
      trap.name = "guillotine";
      have_name = true;
      break;

    case CL_Keyboard::KEY_2:
      trap.name = "hammer";
      have_name = true;
      break;

    case CL_Keyboard::KEY_3:
      trap.name = "spike";
      have_name = true;
      break;

    case CL_Keyboard::KEY_4:
      trap.name = "laser_exit";
      have_name = true;
      break;

    case CL_Keyboard::KEY_5:
      trap.name = "fake_exit";
      have_name = true;
      break;

    case CL_Keyboard::KEY_6:
      trap.name = "smasher";
      have_name = true;
      break;

    default:
      cout << "unknow keypressed" << endl;
    }
  }
  
  return new TrapObj(trap);
}

EditorObj*
ObjectSelector::get_groundpiece()
{
  string str;
  surface_data data;
  data.x_pos = CL_Mouse::get_x() - x_offset;
  data.y_pos = CL_Mouse::get_y() - y_offset;
  
  str = read_string("Input Hotspot gfx:", last_object);
  
  if (str.empty())
    return 0;

  last_object = str;  
  data.res_desc = ResDescriptor("resource:global.dat", str);
  data.res_name = "global.dat";
  data.name = str;

  /*  surf = CL_Res_Surface::load(desc.res_name.c_str(), 
			      *PingusResource::get(desc.filename));
			      */
  return new PSMObj(data);
}

EditorObj*
ObjectSelector::get_hotspot()
{
  string str;
  hotspot_data data;
  data.x_pos = CL_Mouse::get_x() - x_offset;
  data.y_pos = CL_Mouse::get_y() - y_offset;
  data.z_pos = 0;
  
  str = read_string("Input Hotspot gfx:", last_object);
  
  if (str.empty())
    return 0;
  
  last_object = str;
  data.desc = ResDescriptor("resource:global.dat", str);
  
  /*  surf = CL_Res_Surface::load(desc.res_name.c_str(), 
			      *PingusResource::get(desc.filename));
			      */
  return new HotspotObj(data);
}

EditorObj*
ObjectSelector::get_entrance()
{
  entrance_data entrance;
  bool have_name = false;
  entrance.x_pos = CL_Mouse::get_x() - x_offset;
  entrance.y_pos = CL_Mouse::get_y() - y_offset;
  entrance.z_pos = 0;

  CL_Display::clear_display();
  font->print_left(20, 20, "Select an entrance");
  font->print_left(20, 50, "1 - generic");
  font->print_left(20, 70, "2 - woodthing");
  CL_Display::flip_display();

  while (!have_name) {
    switch (read_key()) {

    case CL_Keyboard::KEY_1:
      entrance.type = "generic";
      have_name = true;
      break;

    case CL_Keyboard::KEY_2:
      entrance.type = "woodthing";
      have_name = true;
      break;

    default:
      cout << "unknow keypressed" << endl;
    }
  }
  
  return new EntranceObj(entrance);
}

EditorObj* 
ObjectSelector::get_exit()
{
  string str;
  exit_data data;
  data.x_pos = CL_Mouse::get_x() - x_offset;
  data.y_pos = CL_Mouse::get_y() - y_offset;
  data.z_pos = 0;
  
  str = read_string("Input Hotspot gfx:", last_object);
  
  if (str.empty())
    return 0;
  
  data.desc = ResDescriptor("resource:global.dat", str);
  
  /*  surf = CL_Res_Surface::load(desc.res_name.c_str(), 
			      *PingusResource::get(desc.filename));
			      */
  return new ExitObj(data);
}

EditorObj* 
ObjectSelector::select_obj_type()
{
  CL_Display::clear_display();
  font->print_left(20, 20, "What object do you want?");
  font->print_left(20, 50, "t - Trap");
  font->print_left(20, 70, "g - Groundpiece");
  font->print_left(20, 90, "h - Hotspot");
  font->print_left(20,110, "e - Entrance");
  font->print_left(20,130, "x - Exit");
  CL_Display::flip_display();
    
  while (true) {
    switch (read_key()) {
    case CL_Keyboard::KEY_T:
      return get_trap();
      break;
    case CL_Keyboard::KEY_G:
      return get_groundpiece();
      break;
    case CL_Keyboard::KEY_H:
      return get_hotspot();
      break;
    case CL_Keyboard::KEY_E:
      return get_entrance();
      break;
    case CL_Keyboard::KEY_X:
      return get_exit();
      break;
    }
  }
}

int
ObjectSelector::read_key()
{
  CL_InputBuffer*   key_buffer = CL_Input::keyboards[0]->get_buffer(0);
  CL_Key key;

  key_buffer->clear();

  while (true) { 
    CL_System::keep_alive();
    
    if (key_buffer->peek_key().state != CL_Key::Pressed) {
      key_buffer->get_key();
      continue;
    }
    
    key = key_buffer->get_key();
    cout << "key: " << key.ascii << endl;
    return key.id;
  }
}

string
ObjectSelector::read_string(string description, string def_str)
{
  string str = def_str;
  cout << "Reading string..." << endl;

  CL_Display::clear_display();
  font->print_left(20, 20, description.c_str());
  font->print_left(20, 40, str.c_str());
  CL_Display::flip_display();
  
  CL_InputBuffer*   key_buffer = CL_Input::keyboards[0]->get_buffer(0);
  CL_Key key;

  key_buffer->clear();

  while (true) { 
    CL_System::keep_alive();

    if (key_buffer->peek_key().state != CL_Key::Pressed) {
      key_buffer->get_key();
      continue;
    }

    key = key_buffer->get_key();
  
    if (key.id == CL_Keyboard::KEY_ENTER) {
      break;
    } else if (key.id == CL_Keyboard::KEY_ESCAPE) {
      return "";
    } else if (key.id == CL_Keyboard::KEY_DELETE
	       || key.id == CL_Keyboard::KEY_BACKSPACE) {
      if (!str.empty())
	str = str.substr(0, str.size() - 1); 
    } else if (key.id == CL_Keyboard::KEY_TAB) {
      cout << "Tab completion not implemented" << endl;
    } else {
      if (key.ascii > 0)
	str += key.ascii;
    }

    CL_Display::clear_display();
    font->print_left(20, 20, description.c_str());
    font->print_left(20, 40, str.c_str());
    CL_Display::flip_display();
  }
  key_buffer->clear();
  return str;  
}

/*
$Log: ObjectSelector.cc,v $
Revision 1.5  1999/12/12 03:05:30  grumbel
Added some cvs keywords, misc other fixes

*/

/* EOF */
